==> polymer-resource-pack-0.9.19+1.21.1-sources.jar: eu/pb4/polymer/resourcepack/api/metadata/PackMcMeta.java <==
package eu.pb4.polymer.resourcepack.api.metadata;

import com.google.gson.JsonParser;
import com.mojang.serialization.Codec;
import com.mojang.serialization.JsonOps;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import eu.pb4.polymer.resourcepack.mixin.accessors.PackOverlaysMetadataAccessor;
import eu.pb4.polymer.resourcepack.mixin.accessors.ResourceFilterAccessor;
import java.util.*;
import net.minecraft.class_155;
import net.minecraft.class_2561;
import net.minecraft.class_3264;
import net.minecraft.class_3272;
import net.minecraft.class_6497;
import net.minecraft.class_7084;
import net.minecraft.class_7085;
import net.minecraft.class_8617;

public record PackMcMeta(class_3272 pack, Optional<class_7084> filter, Optional<class_8617> overlays, Optional<LanguageResourceMetadata> language) {
    public static final Codec<PackMcMeta> CODEC = RecordCodecBuilder.create(instaince -> instaince.group(
            class_3272.field_45051.fieldOf("pack").forGetter(PackMcMeta::pack),
            ResourceFilterAccessor.getCODEC().optionalFieldOf("filter").forGetter(PackMcMeta::filter),
            PackOverlaysMetadataAccessor.getCODEC().optionalFieldOf("overlays").forGetter(PackMcMeta::overlays),
            LanguageResourceMetadata.CODEC.optionalFieldOf("language").forGetter(PackMcMeta::language)
    ).apply(instaince, PackMcMeta::new));

    public static PackMcMeta fromString(String string) {
        return CODEC.decode(JsonOps.INSTANCE, JsonParser.parseString(string)).getOrThrow().getFirst();
    }

    public String asString() {
        return CODEC.encodeStart(JsonOps.INSTANCE, this).getOrThrow().toString();
    }

    public static class Builder {
        private class_3272 metadata = new class_3272(
                class_2561.method_43470("Server Resource Pack"),
                class_155.method_16673().method_48017(class_3264.field_14188),
                Optional.empty()
        );
        private final List<class_7085> filter = new ArrayList<>();
        private final List<class_8617.class_8618> overlay = new ArrayList<>();
        private final Map<String, LanguageDefinition> language = new HashMap<>();

        public Builder metadata(class_3272 metadata) {
            this.metadata = metadata;
            return this;
        }

        public Builder description(class_2561 description) {
            this.metadata = new class_3272(description, this.metadata.comp_1581(), this.metadata.comp_1582());
            return this;
        }

        public Builder addFilter(class_7085 entry) {
            this.filter.add(entry);
            return this;
        }

        public Builder addOverlay(class_6497<Integer> format, String overlay) {
            this.overlay.add(new class_8617.class_8618(format, overlay));
            return this;
        }

        public Builder addOverlay(class_8617.class_8618 entry) {
            this.overlay.add(entry);
            return this;
        }

        public Builder addLanguage(String name, LanguageDefinition definition) {
            this.language.put(name, definition);
            return this;
        }

        public PackMcMeta build() {
            return new PackMcMeta(this.metadata,
                    this.filter.isEmpty() ? Optional.empty() : Optional.of(new class_7084(this.filter)),
                    this.overlay.isEmpty() ? Optional.empty() : Optional.of(new class_8617(this.overlay)),
                    this.language.isEmpty() ? Optional.empty() : Optional.of(new LanguageResourceMetadata(this.language))
            );
        }
    }
}

==> polymer-resource-pack-0.14.4+1.21.10-sources.jar: eu/pb4/polymer/resourcepack/api/metadata/PackMcMeta.java <==
package eu.pb4.polymer.resourcepack.api.metadata;

import com.google.gson.JsonParser;
import com.mojang.serialization.Codec;
import com.mojang.serialization.JsonOps;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import eu.pb4.polymer.resourcepack.mixin.accessors.PackOverlaysMetadataAccessor;
import eu.pb4.polymer.resourcepack.mixin.accessors.ResourceFilterAccessor;
import java.util.*;
import net.minecraft.class_11555;
import net.minecraft.class_155;
import net.minecraft.class_2561;
import net.minecraft.class_3264;
import net.minecraft.class_3272;
import net.minecraft.class_6497;
import net.minecraft.class_7084;
import net.minecraft.class_7085;
import net.minecraft.class_8617;

public record PackMcMeta(class_3272 pack, Optional<class_7084> filter, Optional<class_8617> overlays, Optional<LanguageResourceMetadata> language) {
    public static final Codec<PackMcMeta> CODEC = RecordCodecBuilder.create(instaince -> instaince.group(
            class_3272.field_61155.comp_3437().fieldOf("pack").forGetter(PackMcMeta::pack),
            ResourceFilterAccessor.getCODEC().optionalFieldOf("filter").forGetter(PackMcMeta::filter),
            class_8617.field_61145.comp_3437().optionalFieldOf("overlays").forGetter(PackMcMeta::overlays),
            LanguageResourceMetadata.CODEC.optionalFieldOf("language").forGetter(PackMcMeta::language)
    ).apply(instaince, PackMcMeta::new));

    public static PackMcMeta fromString(String string) {
        return CODEC.decode(JsonOps.INSTANCE, JsonParser.parseString(string)).getOrThrow().getFirst();
    }

    public String asString() {
        return CODEC.encodeStart(JsonOps.INSTANCE, this).getOrThrow().toString();
    }

    public static class Builder {
        private class_3272 metadata = new class_3272(
                class_2561.method_43470("Server Resource Pack"),
                new class_6497<>(class_155.method_16673().method_70592(class_3264.field_14188))
        );
        private final List<class_7085> filter = new ArrayList<>();
        private final List<class_8617.class_8618> overlay = new ArrayList<>();
        private final Map<String, LanguageDefinition> language = new HashMap<>();

        public Builder metadata(class_3272 metadata) {
            this.metadata = metadata;
            return this;
        }

        public Builder description(class_2561 description) {
            this.metadata = new class_3272(description, this.metadata.comp_1582());
            return this;
        }

        public Builder addFilter(class_7085 entry) {
            this.filter.add(entry);
            return this;
        }

        public Builder addOverlay(class_6497<class_11555> format, String overlay) {
            this.overlay.add(new class_8617.class_8618(format, overlay));
            return this;
        }

        public Builder addOverlay(class_8617.class_8618 entry) {
            this.overlay.add(entry);
            return this;
        }

        public Builder addLanguage(String name, LanguageDefinition definition) {
            this.language.put(name, definition);
            return this;
        }

        public PackMcMeta build() {
            return new PackMcMeta(this.metadata,
                    this.filter.isEmpty() ? Optional.empty() : Optional.of(new class_7084(this.filter)),
                    this.overlay.isEmpty() ? Optional.empty() : Optional.of(new class_8617(this.overlay)),
                    this.language.isEmpty() ? Optional.empty() : Optional.of(new LanguageResourceMetadata(this.language))
            );
        }

        public class_3272 metadata() {
            return this.metadata;
        }

        public List<class_8617.class_8618> overlays() {
            return this.overlay;
        }
    }
}
